/*
 * Created on 2005.6.19
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

/**
 * @author Betalord
 *
 * TODO To change the template for this generated type comment go to
 * Window - Preferences - Java - Code Style - Code Templates
 */
import java.util.Vector;

public class Battle {
	static int IDCounter; // this is the ID that the next battle will have
	
	public int ID; // each battle has it's own unique ID
	public String title; // description of the battle set by founder of the battle
	public Client founder; // founder or host of the battle
	public Vector clients; // clients without the founder
	public Vector bots;
	public String mapName;
	public int maxPlayers;
	public String password; // use restricted() method to find out if battle is password-protected
	public int port;
	public int metal; // starting metal
	public int energy; // starting energy
	public int units; // max. units
	public int startPos; // 0 = fixed, 1 = random, 2 = choose in game
	public int gameEndCondition; // 0 = game continues if commander dies, 1 = game ends if commander dies
	public int hashCode; // see notes for description!
	public int rank; // if 0, no rank limit is set. If 1 or higher, only players with this rank (or higher) can join the battle (Note: rank index 1 means seconds rank, not the first one, since you can't limit game to players of the first rank because that means game is open to all players and you don't have to limit it in that case)
	public String modName;
	public Vector disabledUnits;
	public StartRect[] startRects;
	public boolean limitDGun;
	
	/* Creates new Battle object from a command that client sent to server. This method
	 * parses the command (String s) and tries to read battle attributes from it. If
	 * unsuccessful, it returns null as a result. */
	public static Battle createBattleFromString(String s, Client founder) {
		String[] parsed = s.split(" ");
		if (parsed.length < 13) return null;
		String[] parsed2 = Misc.makeSentence(parsed, 12).split("\t");
		if (parsed2.length != 3) return null;
		
		String pass = parsed[1];
		if (!pass.equals("*")) if (!Misc.isValidName(pass)) return null;
		
		int port;
		int maxPlayers;
		int startMetal;
		int startEnergy;
		int maxUnits;
		int startPos;
		int gameEndCondition;
		boolean limitDGun;
		int hash;
		int rank;
		
		try {
			port = Integer.parseInt(parsed[2]);
			maxPlayers = Integer.parseInt(parsed[3]);
			startMetal = Integer.parseInt(parsed[4]);
			startEnergy = Integer.parseInt(parsed[5]);
			maxUnits = Integer.parseInt(parsed[6]);
			startPos = Integer.parseInt(parsed[7]);
			gameEndCondition = Integer.parseInt(parsed[8]);
			limitDGun = Misc.strToBool(parsed[9]); 
			hash = Integer.parseInt(parsed[10]);
			rank = Integer.parseInt(parsed[11]);
		} catch (NumberFormatException e) {
			return null; 
		}
		
		if ((startPos < 0) || (startPos > 2)) return null;
		if ((gameEndCondition < 0) || (gameEndCondition > 1)) return null;

		return new Battle(founder, pass, port, maxPlayers, startMetal, startEnergy, maxUnits, startPos, gameEndCondition, limitDGun, hash, rank, parsed2[0], parsed2[1], parsed2[2]);
	}
	
	public Battle(Client founder, String password, int port, int maxPlayers, int startMetal, int startEnergy, int maxUnits, int startPos, int gameEndCondition, boolean limitDGun, int hashCode, int rank, String mapName, String title, String modName) {
		this.ID = IDCounter++;
		this.title = new String(title);
		this.founder = founder;
		this.clients = new Vector();
		this.bots = new Vector();
		this.mapName = new String(mapName);
		this.maxPlayers = maxPlayers;
		this.password = new String(password);
		this.port = port;
		this.metal = startMetal;
		this.energy = startEnergy;
		this.units = maxUnits;
		this.startPos = startPos;
		this.gameEndCondition = gameEndCondition;
		this.limitDGun = limitDGun;
		this.hashCode = hashCode;
		this.rank = rank;
		this.modName = new String(modName);
		this.disabledUnits = new Vector();
		this.startRects = new StartRect[10];
		for (int i = 0; i < startRects.length; i++) startRects[i] = new StartRect();
	}
	
	/* creates BATTLEOPENED command from this battle and returns it as a result */
	public String createBattleOpenedCommand() {
		return "BATTLEOPENED " + ID + " " + founder.account.user + " " + founder.IP + " "+ port + " " + maxPlayers + " " + Misc.boolToStr(restricted()) + " " + rank + " " + mapName + "\t" + title + "\t" + modName;
	}
	
	/* sends series of CLIENTBATTLESTATUS command to client telling him about battle statuses
	 * of all clients in this battle EXCEPT for himself! */
	public void notifyOfBattleStatuses(Client client) {
		for (int i = 0; i < this.clients.size(); i++) 
			if ((Client)this.clients.get(i) == client) continue;
			else client.sendLine("CLIENTBATTLESTATUS " + ((Client)this.clients.get(i)).account.user + " " + ((Client)this.clients.get(i)).battleStatus);
		if (founder != client) client.sendLine("CLIENTBATTLESTATUS " + founder.account.user + " " + founder.battleStatus);
	}
	
	/* notifies all clients in the battle (including the client) about new battle status
	 * of the client. */
	public void notifyClientsOfBattleStatus(Client client) {
		sendToAllClients("CLIENTBATTLESTATUS " + client.account.user + " " + client.battleStatus);
	}
	
	/* sends String s to all clients participating in this battle */
	public void sendToAllClients(String s) {
		for (int i = 0; i < this.clients.size(); i++) {
			((Client)clients.get(i)).sendLine(s);
		}
		founder.sendLine(s);
	}
	
	/* sends String s to all clients participating in this battle except for the founder */
	public void sendToAllClientsExceptFounder(String s) {
		for (int i = 0; i < this.clients.size(); i++) {
			((Client)clients.get(i)).sendLine(s);
		}
	}
	
	private String clientsToString() {
		if (this.clients.size() == 0) return "";
		String s = new String(((Client)clients.get(0)).account.user);
		for (int i = 1; i < this.clients.size(); i++)
			s.concat(" " + ((Client)clients.get(i)).account.user);
		return s;
	}
	
	public boolean addClient(Client client) {
		return this.clients.add(client);
	}
	
	public boolean removeClient(Client client) {
		return this.clients.remove(client);
	}
	
	public boolean restricted() {
		return !password.equals("*");
	}
	
	// returns number of spectators in the battle
	public int spectatorCount() {
		int count = 0;
		for (int i = 0; i < clients.size(); i++)
			if (Misc.getModeFromBattleStatus(((Client)clients.get(i)).battleStatus) == 0) count++;
		if (Misc.getModeFromBattleStatus(founder.battleStatus) == 0) count++;
		return count;
	}
	
	public int nonSpectatorCount() {
		return clients.size()+1-spectatorCount();
	}
	
	public boolean isClientInBattle(Client client) {
		for (int i = 0; i < clients.size(); i++)
			if (((Client)clients.get(i)) == client) return true;
		if (founder == client) return true;	
		
		return false;
	}
	
	public boolean isClientInBattle(String username) {
		for (int i = 0; i < clients.size(); i++)
			if (((Client)clients.get(i)).account.user.equals(username)) return true;
		if (founder.account.user.equals(username)) return true;	
		
		return false;
	}
	
	public Client getClient(String username) {
		for (int i = 0; i < clients.size(); i++)
			if (((Client)clients.get(i)).account.user.equals(username)) return (Client)clients.get(i);
		if (founder.account.user.equals(username)) return founder;	
		
		return null;
	}
	
	/* returns -1 if unit is not in disabled list, index otherwise */
	public int getUnitIndexInDisabledList(String unitname) {
		for (int i = 0; i < disabledUnits.size(); i++)
			if (((String)disabledUnits.get(i)).equals((unitname))) return i;
		return -1;	
	}
	
	public void sendDisabledUnitsListToClient(Client client) {
		if (disabledUnits.size() == 0) return ; // nothing to send
		
		String line = (String)disabledUnits.get(0);
		for (int i = 1; i < disabledUnits.size(); i++)
			line = line.concat(" " + (String)disabledUnits.get(i));
			
		client.sendLine("DISABLEUNITS " + line);
	}
	
	public int getBot(String name) {
		for (int i = 0; i < bots.size(); i++)
			if (((Bot)bots.get(i)).name.equals(name)) return i;
		return -1;	
	}

	/* removes first bot in bots list which is owned by client */
	public boolean removeFirstBotOfClient(Client client) {
		for (int i = 0; i < bots.size(); i++)
			if (((Bot)bots.get(i)).ownerName.equals(client.account.user)) {
				sendToAllClients("REMOVEBOT " + ID + " " + ((Bot)bots.get(i)).name);
				bots.remove(i);
				return true;
			}
		return false;	
	}
	
	/* removes all bots owned by client */
	public void removeClientBots(Client client) {
		while (removeFirstBotOfClient(client)) ;
	}
	
	public void sendBotListToClient(Client client) {
		for (int i = 0; i < bots.size(); i++)
			client.sendLine("ADDBOT " + ID + " " + ((Bot)bots.get(i)).name + " " + ((Bot)bots.get(i)).ownerName + " " + ((Bot)bots.get(i)).battleStatus + " " + ((Bot)bots.get(i)).AIDll);
	}
	
	public void sendStartRectsListToClient(Client client) {
		for (int i = 0; i < startRects.length; i++) {
			if (!startRects[i].enabled) continue; 
			client.sendLine("ADDSTARTRECT " + i + " " + startRects[i].left + " " + startRects[i].top + " " + startRects[i].right + " " + startRects[i].bottom);
		} 
	}
	
	
}
